/*
    File:   prj_01.cpp
    Author: Jonathan Dale Kirwan

    Creation Date:  Wed 03-Oct-2007 18:21:00
    Last Modified:  Wed 03-Oct-2007 18:21:00

    Copyright 2007, 2009, Jonathan Dale Kirwan, jonk@infinitefactors.org

    This source code module is distributed under the terms of the GNU General
    Public License (aka GNU GPL.)

    You should have received a copy of this license as COPYING.  But if not,
    it can currently be accessed over the web at:

        http://www.gnu.org/licenses/gpl.txt

    Or receive a copy by writing to the Free Software Foundation, Inc., the
    current address of which (at this time of writing) is:

        51 Franklin St
        Fifth Floor
        Boston, MA 02110
        USA


    SHORT README FOR THE FIRST PROJECT OF SEVERAL

    Since this is the first project of several, I'd like to say a few things.

    I include all documentation within the source code.  I chose this method
    instead of providing documentation in a README text file or PDF/DOC file
    because it is all too easy for files to become separated.  Also, this way
    strongly encourages me to update the documentation in the same file I am
    busy modifying.

    These projects aren't designed to teach the c or c++ language and I assume
    the reader knows the basic elements already or has access to a teacher or
    friend who can help when needed.

    I delay the introduction of interrupt handling, intentionally.  Those who
    are already very familiar with them may skip over some of the lessons.  I
    wanted to include a more gentle approach that would provide some kind of
    foundation for those not yet used to interrupts, in the earlier lessons,
    so that the introduction of interrupt handling and the experience of using
    them could be a separate lesson.

    These lessons continue into much more than programming alone and deal with
    such things as human visual perceptions.  Hopefully, there's a little
    something for everyone to enjoy.

    In these projects, I will sometimes place long discussions in the code.
    These discussions will be highlighted by "*** READ THIS ***".  It's wise
    to pay particular attention to these sections and read the text there.  In
    later projects, I'll remove comments that duplicate what was already said
    in earlier ones.  So check back on earlier projects, if needed.


    DESCRIPTION

    This is project 1 -- turning the LED on and off.

    Read the comments near the code below and modify the program so that the
    LED is turned on (and the code will properly compile.)  Then follow the
    instructions to load the code into the USB device and instruct the IDE to
    run the program.  The code as it exists at first won't even compile, so
    read the notes and correct the error to make a working program.

    Once you've finished making a version that turns the LED on and made sure
    it works okay, I want you to then add another line after the one that turns
    the LED on, designed to turn the LED back off.  This will happen so fast,
    that you won't see the LED ever come on.  But adding a line, instead of
    just modifying the line, will make sure that you actually do know how to
    turn the LED back off in your code.  If you don't, the LED will stay on.

    So, here are the steps for project 1:

        (1) Modify the existing program so that it compiles okay and so
            that the LED is turned on.  Verify this by loading your code
            into the USB hardware device and letting it run.

        (2) Modify the code from step (1) by adding another line after
            the one that turns the LED on, designed to turn the LED back
            off, again.  Verify this by loading that code into the USB
            hardware device and letting it run.

        (3) Once you've performed steps (1) and (2), this time use the
            same code from step (2), but instead of telling the debugger
            to just "run" your program, this time I want you to use the
            F10 key repeatedly so that you can "single-step" through your
            code and see the LED first turn on, then turn off, but only
            as take each single step through the program.


    TARGET COMPILER

    This module is designed to be compiled with the IAR C++/C compiler that
    comes on a CD with the Texas Instruments "MSP430 Ultra-Low-Power MCUs
    eZ430-F2013 Development Tool."  This IAR compiler is also known as the
    "IAR Kickstart" compiler and it can be downloaded from the web site for
    Texas Instruments, on pages concerning the MSP430 processor.

    IAR provides the Kickstart version of their compiler, but imposes some
    limitations on your code size to encourage you to buy their full compiler
    license if you try and use it for professional work.  But for the purposes
    of education here, the IAR Kickstart tools for the MSP430 are sufficient.
 
 
    MODIFICATIONS
 
    Original source.
*/


/*  ----------------------------------------------------------------------  */
#include <msp430x20x3.h>
/*  ----------------------------------------------------------------------
    *** READ THIS ***
    The above named #include is used to include a special file provided by
    IAR for their Kickstart compiler, used to set up special names that
    are given by Texas Instruments for their MSP430 microcontrollers.
    These are special memory locations and values, unique to these pro-
    cessors and not strictly a part of c++, itself.

    If you look closely at the included filename, you will see that it
    includes some odd number and letter combinations.  This is because
    Texas Instruments gives names to each of their processors and pro-
    cessor families.  An example is the MSP430F2013.  This is a specific
    model of a 'branch' of the MSP430 family.  The branch in this case is
    the MSP430F2xxx, which includes several specific models with different
    numbers instead of the 'xxx' shown above.  The processor used in the
    Development Tool is the MSP430F2013 and IAR has already prepared a
    special file for a part of the branch ending in '3's.  In this case,
    IAR decided to call this file 'msp430x20x3.h', which is meant to be
    used with any MSP430F20x3, with 'x' being any single digit.  Since we
    are using the MSP430F2013, 'x' is 1, which is fine.  This is the right
    file to include.

    If you are interested in the detailed contents of the include file,
    you can go to the installation directory for the IAR MSP430 Kickstart
    software and find and open the file for reading.
*/



/*  ----------------------------------------------------------------------  */
int main( ) {
/*  ----------------------------------------------------------------------
    *** READ THIS ***
    As you are probably aware, c++ programs first handle constructors for
    any static class instances and then begin by running the function,
    main().  This is basically where you get to take control.  (The c++
    library usually includes a small amount of code that executes before-
    hand, to set up anything the library routines may need in order to
    perform well.  But once all that is done [and the details are usually
    completely hidden from you], this is where things begin for you.

    Just in case you wonder and if the above paragraph isn't completely
    clear to you, it is _very_ unlikely that the first lines in the main()
    function represent the first actual instructions that the processor
    begins with.  There is almost _always_ some other code that has been
    prepared by the compiler vendor or someone else who well understands
    the mechanisms required to start a c/c++ program running properly.  If
    the code I'm talking about here, the code that precedes main(), comes
    from the c/c++ compiler vendor, it is often included as something
    called "crt0" or something similar (stands for "c run time 0.")  Some-
    times you can find the source (often assembly code) included in the
    library source distribution from them if you look for a file with that
    name or something similar to it.

    The purpose of the 'crt0' code is somewhat complicated to explain, but
    the basic idea is that the c and c++ languages require certain things
    to be true before main() starts up.  In other words, c/c++ guarantees
    some things to the programmer and a microcontroller's hardware, in and
    of itself, doesn't generally meet those guarantees.  It's the job of
    the crt0 code to tie up loose ends so that code in main() can assume
    the guarantees are, in fact, met.

    An example may help.  If you state "static char myvar= 17;" in your
    source code, c/c++ guarantee that when the program starts up that the
    initial value of 'myvar' is 17.  'myvar' will be located in some RAM
    memory location.  But there is no guarantee that the value will be
    17 when the microcontroller and the rest of the hardware is powered
    up.  It's the job of the crt0 code to ensure that 'myvar' gets set to
    17 before main() starts up.  "crt0" is hidden from you, but it exists
    all the same.
*/

    /*  *** READ THIS ***
        Many microcontrollers have a special device called a "watchdog" timer.
        The watchdog timer is just a counter that counts down or up from some
        value.  When the value gets to zero, the watchdog timer is supposed to
        force the processor to reset or restart itself.  In normal operation,
        there is supposed to be some code that periodically resets the watchdog
        timer so that this count-to-zero never takes place and so that the cpu
        does not experience a reset/restart.  However, if for some reason the
        normal behavior of the program fails and/or it "crashes," then the
        watchdog timer with trigger a reset/restart and force the program back
        to the beginning.

        The watchdog might be something like you could have under the Windows
        operating system -- for example, a hidden program that will cause your
        computer to automatically reboot itself in case of a serious crash.

        In our case here, we don't want any interferences, so the watchdog
        timer is simply and permanently turned off.
    */
        WDTCTL= WDTPW + WDTHOLD;            // turn off the watchdog timer.

    /*  *** READ THIS ***
        The LED on the little USB unit is tied to a special "pin" on the cpu.
        In this case, the pin is called "P1.0" and this just means that the
        pin corresponds to bit 0 of port 1.  These "pins" can be used either
        for reading a binary 0 or 1 value or else for outputting a binary 0 or
        1.  When the cpu is powered up, or booted, the default case sets all
        the pins to "input mode."  (That's somewhat safer.)  But since we want
        to turn on the LED, or turn it off, we need to use that pin for output,
        not for input.  To do that, we need to set a bit in a special memory or
        register called P1DIR, which is the direction control register for port
        1.  A bit value of '1' means output for that pin and a bit value of '0'
        means input for that pin.

        Below, note the use of |=.  This allows us to ignore the direction to
        which the other P1 port pins are currently set.  If we'd just loaded a
        value of '1' into P1DIR, that would have forced all the other pins to
        be inputs.  In this case, they already are and that wouldn't harm
        anything.  But if this code had been in a place where we didn't know
        what the other pins were doing, and we just set the entire value of
        P1DIR to some arbitrary value, we might mess up the prior directions
        that had already been set up elsewhere in the code for the other pins.
        That would not be a good thing.  Best to get into the right practice
        at the outset.  So we use OR to set the direction only for P1.0.
    */
        P1DIR |= 0x01;                      // set P1.0 for output

    /*  *** READ THIS ***
        Now we are ready to turn on the LED.  Either a '1' or a '0' might be
        the right value to turn the LED on.  Which one it actually is depends
        on how the hardware is designed.  In this case, a '1' means ON and a
        '0' means off.  So what we need to do to turn on the LED is to make
        sure that P1.0 = 1.  That will turn it on.

        There is another special register called P1OUT which we need to use
        to set the port pin, as desired.  To change the pin, P1.0, we need to
        change the value of bit 0 of P1OUT (which is 8 bits in size.)

        I've intentionally made things here so that the program won't compile
        until you correct the following line.  However, I've included a copy
        of a line that will work, but as a comment and not actual code.
    */
        // P1OUT |= 0x01;
        P1OUT |=

    /*  *** READ THIS ***
        Another option to try is the following.  What's the difference?  Why
        prefer one or the other?  Think about it!
    */
        // P1OUT= 0x01;

    /*  *** READ THIS ***
        After the LED is turned on, we are done.  However, this is a program
        running on a microcontroller.  We cannot just return from main(),
        because there is no operating system to return to.  So we need to just
        stay in this final spot -- using an infinite loop to just sit.  There
        is no harm in doing that.  The LED is on, and the program just locks
        itself up after that.  So the LED stays on, and nothing much else.
    */
        for ( ; ; )
            ;

    return 0;       /* never reaches this statement! */
}
