/*
    File:   prj_02.cpp
    Author: Jonathan Dale Kirwan

    Creation Date:  Wed 03-Oct-2007 18:20:54
    Last Modified:  Wed 03-Oct-2007 18:20:54

    Copyright 2007, 2009, Jonathan Dale Kirwan, jonk@infinitefactors.org

    This source code module is distributed under the terms of the GNU General
    Public License (aka GNU GPL.)

    You should have received a copy of this license as COPYING.  But if not,
    it can currently be accessed over the web at:

        http://www.gnu.org/licenses/gpl.txt

    Or receive a copy by writing to the Free Software Foundation, Inc., the
    current address of which (at this time of writing) is:

        51 Franklin St
        Fifth Floor
        Boston, MA 02110
        USA


    DESCRIPTION

    This is project 2 -- The purpose is to now make the LED blink on and off
    at a visible rate.

    Read the comments near the code below and modify the program so that the
    LED blinks at an even rate you can see.  The code as it exists at first
    will not produce a visible blinking effect, so you will need to correct
    the error.

    Once you've finished making a version that blinks the LED on and off, try
    different delays and see what happens.

    So, here are the steps for project 2:

        (1) Modify the existing program so that it compiles okay and so
            that the LED blinks at rates you can detect well.  Verify
            this by loading your code into the USB hardware device and
            letting it run.  Read notes for help, too.  If you don't read
            and follow them, the LED won't seem to blink!

        (2) Modify the code from step (1) by changing the delay count.
            Use at least three different values to get a feel for it.
            You can get an idea of the blinking rate for each starting
            delay count by observing the LED for 15 seconds and counting
            the number of ON cycles you see.  Make a table of values,
            showing the count value you used and the number of pulses
            you counted in 15 seconds.  Include at least four entries in
            the table.

        (3) Stay in DEBUG mode and return to the for-loop 'i' variable
            without the volatile qualifier active.  Now, see if you can
            make the LED blink but without using an XOR operation.  You
            are only allowed to set or clear the bit for P1.0, this time.
            No XOR allowed.  How did you modify the code to get similar
            results?


    NOTE

    In these projects, I will sometimes place long discussions in the code.
    These discussions will be highlighted by "*** READ THIS ***".  It's wise
    to pay particular attention to these sections and read the text there.  In
    later projects, I'll remove comments that duplicate what was already said
    in earlier ones.  So check back on earlier projects, if needed.


    TARGET COMPILER

    This module is designed to be compiled with the IAR C++/C compiler that
    comes on a CD with the Texas Instruments "MSP320 Ultra-Low-Power MCUs
    eZ430-F2013 Development Tool."  This IAR compiler is also known as the
    "IAR Kickstart" compiler and it can be downloaded from the web site for
    Texas Instruments, on pages concerning the MSP430 processor.

    IAR provides the Kickstart version of their compiler, but imposes some
    limitations on your code size to encourage you to buy their full compiler
    license if you try and use it for professional work.  But for the purposes
    of education here, the IAR Kickstart tools for the MSP430 are sufficient.
 
 
    MODIFICATIONS
 
    Original source.
*/


/*  ----------------------------------------------------------------------  */
#include <msp430x20x3.h>
/*  ----------------------------------------------------------------------
    Include definitions appropriate for the MSP430F2013 microcontroller.
    It is part of the compiler vendor's product, designed to describe key
    datasheet details, and was not written separately for this project.

    See the file itself for details, under the installation directory for
    the IAR compiler toolset.
*/


/*  ----------------------------------------------------------------------  */
int main( ) {
/*  ----------------------------------------------------------------------
    Project 2 starts here.  See discussion in the file header for detailed
    information about the project.  For more discussion of the first two
    statements in this code also see Project 1.
*/

    /*  Turn the watchdog timer off.
    */
        WDTCTL= WDTPW + WDTHOLD;

    /*  Enable port pin P1.0 for output (P1.0 is the LED pin.)
    */
        P1DIR |= 0x01;

    /*  *** READ THIS ***
        Now we are ready to blink the LED.  Since we are going to keep on
        doing this forever, we can use a special for() loop for this purpose.

        Normally, the MSP430 microcontroller is so fast that just flipping the
        LED on and then off and then on, and so on, would mean it would blink
        at super-human rates.  You'd never notice.  So we need to slow it down.
        To do that, we are going to use a delay loop -- often known as a "busy
        wait" loop, because it just sits busy doing nothing in order to delay
        long enough.  This is the easy way to do this.  There are better ways,
        which allow the time to be spent doing something else useful, but this
        is quite good enough for our purposes here.

        The following code will NOT seem to blink the LED, unless you modify
        it.  The reason is the number 5 below.  It presents far, far too short
        of a delay.  You will need to change the value into the tens of
        thousands to make it work better.  Try 40000 or 60000, for example.
        Then examine the code carefully to make sure you understand why it
        does what it does, for you.  Why the XOR, for example?  And why use
        the 'volatile' keyword?  (The teacher can explain this.)
    */
        for ( ; ; ) {
            P1OUT ^= 0x01;              // using XOR toggles the pin value.
            for ( unsigned int i= 5; i != 0; --i )
                ;
        }

    /*  *** READ THIS ***
        Note that we don't need one of those "forever loops" down here.  That
        is because it is already present, in the above code.  The return
        statement below will not be executed.
    */

    return 0;
}
