/*
    File:   prj_03.cpp
    Author: Jonathan Dale Kirwan

    Creation Date:  Wed 03-Oct-2007 18:20:54
    Last Modified:  Wed 03-Oct-2007 18:20:54

    Copyright 2007, 2009, Jonathan Dale Kirwan, jonk@infinitefactors.org

    This source code module is distributed under the terms of the GNU General
    Public License (aka GNU GPL.)

    You should have received a copy of this license as COPYING.  But if not,
    it can currently be accessed over the web at:

        http://www.gnu.org/licenses/gpl.txt

    Or receive a copy by writing to the Free Software Foundation, Inc., the
    current address of which (at this time of writing) is:

        51 Franklin St
        Fifth Floor
        Boston, MA 02110
        USA


    DESCRIPTION

    This is project 3 -- The purpose is to learn about the 'volatile' keyword.

    Read the comments near the code below and modify the program so that the
    LED blinks at an even rate you can see.  The code as it exists at first
    will not produce a visible blinking effect, so you will need to correct
    the error.

    Once you've finished making a version that blinks the LED on and off, try
    different delays and see what happens.

    So, here are the steps for project 3:

        (1) This program starts out similar to Project 2's final state,
            with about 30 blinks in 15 seconds.  First verify this result
            by compiling this code and testing it.  Then modify this project
            from DEBUG to RELEASE by selecting the project in the workspace
            and using the menu option Project/Edit Configurations... and
            then recompile and test the program.  If you do this correctly,
            the LED will appear to turn on and not blink at all!  Why?  And
            is it really just blinking fast?  Or only on?

            (The compiler's fancy optimizer sees that the for-loop does
            nothing useful and completely eliminates it when RELEASE is
            selected.  In DEBUG mode, it doesn't try to be that 'smart' and
            so it doesn't look close enough at the for-loop to see that
            nothing happens in the loop.)

        (2) Staying in RELEASE mode for this project, now find the word
            "volatile" that is commented out in the for-loop and remove
            the comment bracketing so that it is active.  Now compile and
            test the project, again.  The LED should start blinking, now.
            But somewhat slower than when using DEBUG mode and without
            the word "volatile" active.  Why?  (The difference is because
            the compiler, even in DEBUG mode but without the volatile
            qualifier, can place the 'i' variable in a register.  When
            the volatile qualifier is active the 'i' variable is located
            in physical memory, instead.  That is somewhat slower to
            access than the register, so the loop runs slower.  Also,
            because 'i' is volatile, the compiler isn't allowed to
            remove the loop, as it did in RELEASE mode when the 'i'
            variable wasn't volatile-qualified.)

        (3) Now switch back to DEBUG mode and keep the word "volatile"
            active.  Compile and test the project.  The LED should still
            blink at just the same rate as in step (4).  Note now that
            there is no difference in the blinking rate, whether compiled
            as RELEASE or DEBUG.

            (These recent steps are just to show that it is possible for
            quite different behaviors in code that otherwise appears about
            the same.  Be aware!)


    NOTE

    In these projects, I will sometimes place long discussions in the code.
    These discussions will be highlighted by "*** READ THIS ***".  It's wise
    to pay particular attention to these sections and read the text there.  In
    later projects, I'll remove comments that duplicate what was already said
    in earlier ones.  So check back on earlier projects, if needed.


    TARGET COMPILER

    This module is designed to be compiled with the IAR C++/C compiler that
    comes on a CD with the Texas Instruments "MSP320 Ultra-Low-Power MCUs
    eZ430-F2013 Development Tool."  This IAR compiler is also known as the
    "IAR Kickstart" compiler and it can be downloaded from the web site for
    Texas Instruments, on pages concerning the MSP430 processor.

    IAR provides the Kickstart version of their compiler, but imposes some
    limitations on your code size to encourage you to buy their full compiler
    license if you try and use it for professional work.  But for the purposes
    of education here, the IAR Kickstart tools for the MSP430 are sufficient.
 
 
    MODIFICATIONS
 
    Original source.
*/


/*  ----------------------------------------------------------------------  */
#include <msp430x20x3.h>
/*  ----------------------------------------------------------------------
    Include definitions appropriate for the MSP430F2013 microcontroller.
    It is part of the compiler vendor's product, designed to describe key
    datasheet details, and was not written separately for this project.

    See the file itself for details, under the installation directory for
    the IAR compiler toolset.
*/


/*  ----------------------------------------------------------------------  */
int main( ) {
/*  ----------------------------------------------------------------------
    Project 3 starts here.  See discussion in the file header for detailed
    information about the project.  For more discussion of the first two
    statements in this code also see Project 1.
*/

    /*  Turn the watchdog timer off.
    */
        WDTCTL= WDTPW + WDTHOLD;

    /*  Enable port pin P1.0 for output (P1.0 is the LED pin.)
    */
        P1DIR |= 0x01;

    /*  *** READ THIS ***
        Compilers are allowed to "optimize away" code, if they can prove that
        eliminating the code still produces the same results (well, the same
        according to some definitions of 'same.')  In step (1), the compiler
        will be permitted to obliterate the inner for-loop.  In doing so,
        the delay will disappear and the outer for-loop will execute each
        iteration far, far faster than the human eye can detect.

        There is a special keyword that is commented out below and which may
        be new to some.  After this project is complete, it might be useful
        to study some explanations available over the web about the keyword.
    */
        for ( ; ; ) {
            P1OUT ^= 0x01;              // using XOR toggles the pin value.
            for ( /* volatile */ unsigned int i= 60000; i != 0; --i )
                ;
        }

    return 0;
}
