/*
    File:   prj_04.cpp
    Author: Jonathan Dale Kirwan

    Creation Date:  Wed 03-Oct-2007 18:20:31
    Last Modified:  Wed 03-Oct-2007 18:20:31

    Copyright 2007, 2009, Jonathan Dale Kirwan, jonk@infinitefactors.org

    This source code module is distributed under the terms of the GNU General
    Public License (aka GNU GPL.)

    You should have received a copy of this license as COPYING.  But if not,
    it can currently be accessed over the web at:

        http://www.gnu.org/licenses/gpl.txt

    Or receive a copy by writing to the Free Software Foundation, Inc., the
    current address of which (at this time of writing) is:

        51 Franklin St
        Fifth Floor
        Boston, MA 02110
        USA


    DESCRIPTION

    This is project 4 -- learn about adjusting cpu clock rates.

    In the previous project, you used a "busy wait" loop to set the delay time
    between LED on/off changes.  There is problem with using a "busy wait" loop
    that depends on such a simple count-down or count-up method.  What if the
    microcontroller was running faster or slower?

    Don't imagine that they all run at the same speed! Just like IBM PCs, which
    have 800MHz versions and 3GHz versions and will run games better or worse,
    depending on how high their overall performance is; the same thing applies
    to microcontrollers, as well.  Electronics designers can arrange things so
    that some of them run faster than others -- even if they are the exact same
    microcontroller, in fact.  So if you use a counter to set some delay, it's
    almost certain that it won't run the same way on every microcontroller, or
    even the same on the very same microcontroller in the very same product.
    Electronics designers are free to set the speed as they like.  And in some
    cases, even software programmers can make changes in how fast it runs.  In
    fact... on this MSP430F2013, we have the ability to do exactly that -- we
    can change how fast it runs.  We are going to experiment with that, now.

    So, here are the steps for project 4:

        (1) Compile the initial version included here and verify that
            the LED blinks at rates you can detect well.  I've put in
            a counter value of 50000, so it should blink about once
            per second, give or take a little bit.  Verify this by
            loading your code into the USB hardware device and letting
            it run.  All this should be about what you'd expect.

        (2) I've included some code in the main() function that is
            commented out, so that it doesn't get used.  Remove the
            two comment characters at the beginning of those two lines
            and recompile the program and download it into the unit.
            Run it.  You should see a significant change in the blink-
            ing rate of the LED.  The reason is that I've increased the
            speed of the microcontroller by changing a couple of special
            registers that exist for that purpose.  This feature isn't
            always present on microcontrollers.  Some have features
            like this and some don't.  So don't assume these things.
            We are just lucky, with the MSP430F2013.

        (3) I've also included two other commented-out lines.  Those
            are located at the very bottom of the source code here.
            Those two lines cause the processor to run at its very
            fastest speed.  Not at a calibrated 16MHz, but faster.
            The actual speed will vary from unit to unit, because of
            variations in manufacturing tolerances.  [Which explains
            why each of these MSP430F2013 microcontrollers from Texas
            Instruments includes the calibration values that we've 
            already tried using in step (2).]  However, if flat-uot
            speed is all you care about, then those two lines will do
            it.  So replace the two lines we enabled in step (2) with
            the two lines at the end of the source code, uncomment
            them so that the compiler will apply them, and then try
            again to compile, load, and run the code.  See if you can
            notice a further difference in the blinking rate of the
            LED.


    NOTE

    In these projects, I will sometimes place long discussions in the code.
    These discussions will be highlighted by "*** READ THIS ***".  It's wise
    to pay particular attention to these sections and read the text there.  In
    later projects, I'll remove comments that duplicate what was already said
    in earlier ones.  So check back on earlier projects, if needed.


    TARGET COMPILER

    This module is designed to be compiled with the IAR C++/C compiler that
    comes on a CD with the Texas Instruments "MSP320 Ultra-Low-Power MCUs
    eZ430-F2013 Development Tool."  This IAR compiler is also known as the
    "IAR Kickstart" compiler and it can be downloaded from the web site for
    Texas Instruments, on pages concerning the MSP430 processor.

    IAR provides the Kickstart version of their compiler, but imposes some
    limitations on your code size to encourage you to buy their full compiler
    license if you try and use it for professional work.  But for the purposes
    of education here, the IAR Kickstart tools for the MSP430 are sufficient.
 
 
    MODIFICATIONS
 
    Original source.
*/


/*  ----------------------------------------------------------------------  */
#include <msp430x20x3.h>
/*  ----------------------------------------------------------------------
    Include definitions appropriate for the MSP430F2013 microcontroller.
    It is part of the compiler vendor's product, designed to describe key
    datasheet details, and was not written separately for this project.

    See the file itself for details, under the installation directory for
    the IAR compiler toolset.
*/


/*  ----------------------------------------------------------------------  */
int main( ) {
/*  ----------------------------------------------------------------------
    Project 4 starts here.  See discussion in the file header for detailed
    information about the project.
*/

    /*  Turn the watchdog timer off.
    */
        WDTCTL= WDTPW | WDTHOLD;

    /*  Enable port pin P1.0 for output (P1.0 is the LED pin.)
    */
        P1DIR |= 0x01;

    /*  *** READ THIS ***
        The following two statements with change the microcontroller's speed.
        There are some specially calibrated values that Texas Instruments puts
        into special memory locations.

        In the following two program lines, I am using those special values to
        help set up a "precise" 16MHz operation.  Some other values are also
        available for slower speeds, too.  You will have to look those up in
        the include file mentioned above to see what they are.  But I'm making
        the microcontroller run faster, for now, using these.

        (Normally, when the "micro" first starts up, it uses default values
        that set its speed up much slower -- roughly 16 times slower, in fact.
        We've been using that in past projects, simply by the fact that we
        didn't take any action there.  So this should speed things up a bit.

        These are commented out, at first.  Uncomment these for 'step 2'.
    */
        // BCSCTL1= CALBC1_16MHZ;
        // DCOCTL= CALDCO_16MHZ;

    /*  Now we are ready to blink the LED.  Since we are going to keep on
        doing this forever, we can use a special for() loop for this purpose.
    */
        for ( ; ; ) {
            P1OUT ^= 0x01;              // using XOR toggles the pin value.
            for ( volatile unsigned int i= 40000; i != 0; --i )
                ;
        }

    return 0;
}

    /*  *** READ THIS ***
        Uncomment and move these to the appropriate place, for 'step 3'.
    */
        // BCSCTL1= XT2OFF | RSEL3 | RSEL2 | RSEL1 | RSEL0;
        // DCOCTL= DCO2 | DCO1 | DCO0;
