/*
    File:   prj_11.cpp
    Author: Jonathan Dale Kirwan

    Creation Date:  Wed 03-Oct-2007 18:20:07
    Last Modified:  Wed 03-Oct-2007 18:20:07

    Copyright 2007, 2009, Jonathan Dale Kirwan, jonk@infinitefactors.org

    This source code module is distributed under the terms of the GNU General
    Public License (aka GNU GPL.)

    You should have received a copy of this license as COPYING.  But if not,
    it can currently be accessed over the web at:

        http://www.gnu.org/licenses/gpl.txt

    Or receive a copy by writing to the Free Software Foundation, Inc., the
    current address of which (at this time of writing) is:

        51 Franklin St
        Fifth Floor
        Boston, MA 02110
        USA


    DESCRIPTION

    This is Project 11 -- Causing the LED to adjust its brightness up and down
    in a continuous, ramping fashion.

    In Project 10, the duty cycle was fixed and set manually.  This time, we
    cause the program to change the duty cycle on its own, proceeding from a
    minimum value to a maximum value that we'll set up.  And then back down
    again.  The LED should then appear to grow brighter and dimmer.  At least,
    in theory.  Practice is often another thing.

    If you recall, Project 10 used one compare register (TACCR0) to set the
    upper limit of the counter used in up/down mode and the PWM period.  The
    other compare register (TACCR1) was used to set the duty cycle by picking
    a value somewhere between 0 and the value in TACCR0.  The interrupt
    routine then did the work.  TACCR1 was configured at compile-time and
    didn't change on its own.  This time we write some code to change it as
    the program continues running.

    The problem here is to vary the LED intensity smoothly up and down by
    changing the TACCR1 value at a programmed rate.  The easiest idea is to
    just add or subtract a small, fixed value every so often.  (Increasing
    the value of TACCR1 lowers the brightness; decreasing TACCR1 raises the
    brightness.)

    Here are the steps for project 11:

        (1) Compile the initial version included here and verify that
            the LED appears to do something other than sit there with
            a single brightness.  The purpose of this project is to
            get the LED to gradually increase in brightness up to some
            point, and then gradually decrease in brightness down to
            some other point (off?)  Is it doing that?

        (2) If it worked as it should have, it seems to dim gradually
            but brighten back up very sharply.  If you carefully read
            through the code, it will probably seem correct, too.  But
            there is a hidden problem that often crops up when working
            with hardware peripherals.  Read the TIMER A2 PROBLEM DETAIL
            section, now.  Then go modify the interrupt code so that the
            commented code there is no longer commented out.  (Include
            it, now.)  Recompile and test.  Oh, did you miss it?  There
            is also a #define that is commented out, above the interrupt
            routine.  Uncomment that, too.  Now it should compile.

        (3) Now for a big step.  Let's move all that code that's clogging
            up our main code back into the interrupt routine and free up
            the main code from doing all that LED maintenence work.  This
            isn't too hard.  Assuming you've completed step (2), here are
            the steps:
            (a) Grab a copy of all of the main code marked as
                *** FROM HERE *** all the way to *** TO HERE ***
                and use it to replace the single line in the
                interrupt code that reads "++counter;"
            (b) Now remove the main code you just copied into
                the interrupt routine and replace it with the
                single line, "__low_power_mode_0( );"  This is
                then look quite familiar -- see Project 10, for
                example.
            (c) Returning to the massive change in the interrupt
                code from step (3a), find these two lines:
                    auto unsigned int tac= TACCR1;
                    static enum { brighter, dimmer } mode= dimmer;
                and move them to the beginning of the interrupt
                function so that they read like this:
                    auto unsigned int tac;
                    static enum { brighter, dimmer } mode= dimmer;
            (d) Also, kill the "for ( ; ; ) {" line.  It isn't
                needed anymore.
            (e) Remove this from the new interrupt code, as well:
                    for ( icounter= 0; icounter < 4; )
                        ;
            (f) Find and comment out the following line:
                    static volatile unsigned int icounter;
            (g) Returning to the interrupt code, insert this line
                just after the "case TAIFG_EVENT:" line:
                    tac= TACCR1;
            (h) Now, you should see the following near the end of
                the interrupt function:
                    }                       // *** TO HERE ***
                        break;
                    }
                Remove the entire line where you see the first "}".
                The result should now look like:
                        break;
                    }
                That should do it!
            Now recompile the code and see what happens!  It should be
            running about four times faster, up and down.  The reason
            is that we've removed 'icounter', which used to cause a
            change to TACCR1 only once every four TAIFG events.  Now,
            we are changing TACCR1 every TAIFG event.  But something
            else should be very prominent, now.  The 'bouncing effect'
            of the LED near its minimum brightness.

            If you didn't get the interrupt code nailed down on your
            own, I've included a copy of it at the very end of this
            file that shows the last part of the final code.  Hopefully,
            your work and mine are close.

        (4) Read TIMER A2 SUCCESS DETAIL to see why step (2) works.
            Now feel free to modify ADJRATE and LIMIT and LEDRATE and
            SYSCLK and recompile to see effects you'd like to try out.

    A final note.  Even after step (2), when the code appears to be working
    reasonably close to what was intended, if you look closely and watch, you
    will see that the LED appears to have a very short, fast dip to OFF and
    then "bounce" back towards brighter values.  The appearance nearer the
    maximum brightness seems more gradual, by comparison.  This is due to the
    logarithm behavior suggested by the Weber-Fechner law discussed below and
    in Project 10.  If you have a plotting/math program (like Excel), you can
    insert a triangle ramp that spans from some minimum to a maximum that is,
    say, 20 times larger.  Then take the logarithm of that triangle ramp and
    display them together.  You should see the effect in the log curve of the
    linear ramp that goes up and down.


    TIMER A2 PROBLEM DETAIL

    Recall the diagram from Project 10 (I've included it here.)  We need to
    consider two cases to see the problem.  The first case to think about will
    be when TACCR1 is being increased, so that it rises upwards towards our
    LIMIT value that is very near to TACCR0.  The following diagram shows this
    case.

            TACCR0 ---->   ^             ^             ^
                          / \           / \           / \        Up-Down
                         /   \         /   \         /   \       Counter
                        /     \       /     \ ....../.....\...   Diagram
                 \    1/       \    3/       \.    /       \
            TACCR1\.../.........\.../.........\   /         \
                   \ /          2\ /          4\ /           \
            ZERO    v             v             v

                 -----|---------|---|---------|-----|-----|---
                      C         C   C         C     C     C
                      C         C   C         C     C     C      Event Names
                      I         I   I         I     I     I      as Described
                      F         F   F         F     F     F      in the User's
                      G         G   G         G     G     G      Guide from TI
                      1         1   1         1     1     1
                 -----|---------|---|---------|-----|-----|---

            ON        ,---------,   ,---------,     ,-----,
                      |         |   |         |     |     |      LED
                      |         |   |         |     |     |      State
            OFF    ---'         '---'         '-----'     '---

            time ----> ----> ----> ----> ----> ----> ----> ---->

    We count four CCIFG1 interrupt events with 'icounter' (these appear in the
    diagram) and then change the value of TACCR1 upwards (which reduces the
    LED-ON duration, dimming it.)  Note that the timer_A2 hardware counter is
    still moving in a downward direction at the time when the code changes
    TACCR1.  The next cycle starts with a raised TACCR1, which works fine.
    This is why the LED appears to dim smoothly.  Things are working as we had
    intended.

    Now, let's examine the case where the value of TACCR1 is being decreased,
    so that it falls downwards towards our LIMIT value that is very near zero.
    The following diagram shows this case.

            TACCR0 ---->   ^             ^             ^
                         1/ \          3/ \           / \        Up-Down
            TACCR1 ....../...\........./...\         /   \       Counter
                        /    2\       /    4\......./.....\...   Diagram
                 \     /       \     /       \     /       \
                  \   /         \   /         \   /         \
                   \ /           \ /           \ /           \
            ZERO    v             v             v

                 --------|---|---------|---||-------|-----|---
                         C   C         C   CC       C     C
                         C   C         C   CC       C     C      Event Names
                         I   I         I   II       I     I      as Described
                         F   F         F   FF       F     F      in the User's
                         G   G         G   GG       G     G      Guide from TI
                         1   1         1   11       1     1
                 --------|---|---------|---||-------|-----|---

            ON           ,---,         ,---,,-------,     ,---
                         |   |         |   ||       |     |      LED
                         |   |         |   ||       |     |      State
            OFF    ------'   '---------'   ''       '-----'

            time ----> ----> ----> ----> ----> ----> ----> ---->

    Again, we count four CCIFG1 interrupt events with 'icounter' (these also
    appear in this new diagram) and then change the value of TACCR1 downwards
    (which increases the LED-ON duration, brightening it.)  Note again that
    the timer_A2 hardware counter is moving in a downward direction at the
    time when the code changes TACCR1.  However... the downward hardware
    counter almost immediately encounters the new TACCR1 value and fires off
    another interrupt, almost immediately.  This is NOT what was intended.

    As a result, while it does appear to work correctly in one direction; it
    does not do so in the other direction.  We need to fix this problem.


    TIMER A2 SUCCESS DETAIL

    To make a success of our approach, we need a consistent, countable way to
    move TACCR1 up and down.  We could adjust things so that we don't move
    TACCR1 downwards when the hardware counter is in a downward counting
    direction.  This would require us to keep track of the hardware counter
    direction and some other software to choose between one moment to change
    TACCR1 and another.  But... there is another method and I chose it here.

    It turns out that there is another event that timer_A2 creates for us,
    based upon when the hardware counter hits zero and turns around.  It is
    called a TAIFG event.  In Project 10 and in this one, it's been happening
    even though we didn't use it for anything before.  This is part of why
    there is an "if" statement in the interrupt code of Project 10 to check
    for the type of interrupt event and to make sure it is a CCIFG1 event.
    Now, we will explicitly use it.

    Here's how Timer A2 is being used.  The upper part of the diagram helps
    show the timer counter's up-down motion, over time.  The line made with
    text, showing peaks and valleys, represents the timer counter, itself, as
    it counts upwards and then downwards, etc.  The timer automatically starts
    counting upwards when the counter reaches zero and automatically starts
    counting downwards when the counter reaches a value that is set into
    TACCR0.  (In these recent projects, TACCR0 is set just once and isn't
    adjusted afterwards.)  It sets the basic cycle time for the LED pulsing.
    However, TACCR1 is moved around in this project in order to cause the LED
    to be on and off for different periods of time.

    That refresher said, here's the new diagram:

            TACCR0 ---->   ^             ^             ^
                          / \           / \           / \        Up-Down
                         /   \         /   \         /   \       Counter
            TACCR1 ...../.....\......./.....\......./.....\...   Diagram
                 \     /       \     /       \     /       \
                  \   /         \   /         \   /         \
                   \ /           \ /           \ /           \
            ZERO    v             v             v

          icounter  1             2             3

                 ---|---|-----|---|---|-----|---|---|-----|---
                    T   C     C   T   C     C   T   C     C
                    A   C     C   A   C     C   A   C     C      Event Names
                    I   I     I   I   I     I   I   I     I      as Described
                    F   F     F   F   F     F   F   F     F      in the User's
                    G   G     G   G   G     G   G   G     G      Guide from TI
                        1     1       1     1       1     1
                 ---|---|-----|---|---|-----|---|---|-----|---

            ON          ,-----,       ,-----,       ,-----,
                        |     |       |     |       |     |      LED
                        |     |       |     |       |     |      State
            OFF    -----'     '-------'     '-------'     '---

            time ----> ----> ----> ----> ----> ----> ----> ---->

    The middle part of the diagram just shows when certain named interrupt
    events take place.  I've included the changes to 'icounter', which is
    now based only upon TAIFG events.  These events are consistent and won't
    be fooled by our changing TACCR1's value.  It's because of this, that we
    get a consistent behavior, up or down.

    This kind of attention to detail is required of most embedded programmers
    if they are to be successful.  Many times, what appears to be both simple
    and accurately written code -- even after reading it over and over again
    and having others look at it with you -- fails completely.  And it does
    so because there is some hardware detail that was missed in the earlier
    evaluation.  Every detail, no matter how trivial, needs to be included
    when you are tracking down a problem behavior that escapes detection.

    This is one of a number of reasons why embedded programming is very, very
    hard for some people.  Those with a penchant for detail tend to do better.
    However, that's only one facet.  A good imagination is another.  Skills at
    mathematics are another.  Knowing how hardware works and is designed is
    still another.  And so on.  But this project provides an object lesson for
    one of several important proclivities of a good embedded programmer.


    TALBOT-PLATEAU LAW and the WEBER-FECHNER LAW

    This law in the field of visual perception states that if a light source
    blinks fast enough that it passes beyond the critical flicker fusion
    frequency (CFF), then the brightness will appear to be the same "as if"
    the light source were steadily operated at the time-averaged luminance.
    In other words, if you operate the light source at twice the luminance
    but only half the time (50% on and 50% off, flickering faster than the
    CFF), then it will appear to have the same luminance as that similar
    light source operated at the lower luminance.

    Keep in mind that operating a light at a 50% duty cycle, above the CFF,
    does not mean that it will appear half as bright as one operated 100%
    of the time.  It means that operating a light at 50% duty cycle and faster
    than the CFF will make it appear the same brightness as another operating
    at half the luminence.  There is a difference in meaning.  Graple with it.

    The reason it doesn't look "half as bright" when operating at a 50% duty
    cycle is that the human eye works according to another law, the Weber-
    Fechner law.  That one states that intensity goes as the logarithm of the
    physical stimuli.


    TARGET COMPILER

    This module is designed to be compiled with the IAR C++/C compiler that
    comes on a CD with the Texas Instruments "MSP320 Ultra-Low-Power MCUs
    eZ430-F2013 Development Tool."  This IAR compiler is also known as the
    "IAR Kickstart" compiler and it can be downloaded from the web site for
    Texas Instruments, on pages concerning the MSP430 processor.

    IAR provides the Kickstart version of their compiler, but imposes some
    limitations on your code size to encourage you to buy their full compiler
    license if you try and use it for professional work.  But for the purposes
    of education here, the IAR Kickstart tools for the MSP430 are sufficient.
 
 
    MODIFICATIONS
 
    Original source.
*/


/*  ----------------------------------------------------------------------  */
#include <msp430x20x3.h>
/*  ----------------------------------------------------------------------
    Include definitions appropriate for the MSP430F2013 microcontroller.
    It is part of the compiler vendor's product, designed to describe key
    datasheet details, and was not written separately for this project.

    See the file itself for details, under the installation directory for
    the IAR compiler toolset.
*/


/*  ----------------------------------------------------------------------  */
#define ADJRATE     (0.02)
/*  ----------------------------------------------------------------------
    This value selects percent change in the value of TACCR1, per unit
    time.  0.02 represents 2%.
*/


/*  ----------------------------------------------------------------------  */
#define LIMIT       (0.002)
/*  ----------------------------------------------------------------------
    This value sets the minimum and maximin values for TACCR1 as a percent
    of the total range.  For example, 0.002 represents 0.2%, which means
    that the lower limit is 0.2% and the upper limit is 99.8%.  ADJRATE
    is how fast TACCR1 is moved between these limits.
*/


/*  ----------------------------------------------------------------------  */
#define LEDRATE     (100)           // in Hertz
/*  ----------------------------------------------------------------------
    Choose some basic LED pulse rate here.  I recommend something at 70Hz
    or faster, since the LED shouldn't appear to the eye as blinking.  But
    other rates can be chosen.
*/


/*  ----------------------------------------------------------------------  */
#define SYSCLK      (16000000)      // in Hertz
/*  ----------------------------------------------------------------------
    Defines the basic SYSCLK rate we want to use here.  Use calibrated
    values only, unless you are willing to set up the DCO and BC1 values
    and create the appropriate SYSCLK value for them.  For now, an error
    is generated if SYSCLK isn't one of the standard calibration values.

    If you want to add an uncalibrated mode here, you need to provide some
    definitions in the #else clause where the #error statement is found.
*/
#if SYSCLK == 16000000
# define DCO         (CALDCO_16MHZ)
# define BC1         (CALBC1_16MHZ)
#elif SYSCLK == 12000000
# define DCO         (CALDCO_12MHZ)
# define BC1         (CALBC1_12MHZ)
#elif SYSCLK == 8000000
# define DCO         (CALDCO_8MHZ)
# define BC1         (CALBC1_8MHZ)
#elif SYSCLK == 1000000
# define DCO         (CALDCO_1MHZ)
# define BC1         (CALBC1_1MHZ)
#else
# error Uncalibrated SYSCLK requested.
#endif


/*  ----------------------------------------------------------------------
    COMPUTE USEFUL PREPROCESSOR SYMBOLS
    ----------------------------------------------------------------------
    Don't change the following preprocessor statements unless you know
    what you are doing and understand their current design.  They are
    here to compute proper programming values for registers used in
    configuring the timer_A2 device.  An error is generated if the
    LEDRATE and SYSCLK cannot be simultaneously accomodated.

    The purpose of each is as follows:
        TICKS       number of SYSCLK ticks per LED pulse period
        IDVAL       the SYSCLK pre-divisor to use; must be power of 2
        IDCTL       the TACCTL field value needed to achieve IDVAL
        TAC         1 plus the required TACCR0 value
*/
#define TICKS       (SYSCLK/2/LEDRATE)
#define IDVAL       (1+(TICKS >> 16))
#if IDVAL > 4
# undef IDVAL
# define IDVAL      (8)
# define IDCTL      (ID_3)
#elif IDVAL > 2
# undef IDVAL
# define IDVAL      (4)
# define IDCTL      (ID_2)
#elif IDVAL == 2
# define IDCTL      (ID_1)
#elif IDVAL == 1
# define IDCTL      (ID_0)
#else
# error LEDRATE is negative; try a positive value.
#endif
#if (TICKS/IDVAL) > 65535
# error LEDRATE is too small for the given SYSCLK; try a lower SYSCLK.
#endif
#define TAC         ((unsigned int) (TICKS/IDVAL))
#define PROXIMITY   ((unsigned int) (TICKS*LIMIT/IDVAL))
#define TACADJ      ((unsigned int) (TICKS*ADJRATE/IDVAL))


static volatile unsigned int icounter;

/*  ----------------------------------------------------------------------  */
int main( ) {
/*  ----------------------------------------------------------------------
    Project 11 starts here.  See discussion in the file header for detailed
    information about the project.
*/

    /*  Turn the watchdog timer off.
    */
        WDTCTL= WDTPW | WDTHOLD;

    /*  Enable port pin P1.0 for output (P1.0 is the LED pin.)
    */
        P1OUT &= ~0x01;
        P1DIR |= 0x01;

    /*  Set up the processor to run at a calibrated rate.
    */
        BCSCTL1= BC1;
        DCOCTL= DCO;

    /*  Set up the timer.
    */
        TACTL= MC_0 | TASSEL_2 | IDCTL | TACLR;             // reset it, first.
        TACCTL0= CM_0 | CCIS_2 | SCS | OUTMOD_1;            // set the mode.
        TACCTL1= CM_0 | CCIS_2 | SCS | OUTMOD_1 | CCIE;
        TACCR0= TAC - 1;
        TACCR1= TAC - PROXIMITY - 1;
        TACTL= MC_3 | TASSEL_2 | IDCTL | TAIE;              // turn it on.

    /*  Enable interrupt events, globally, and go to sleep.
    */
        __enable_interrupt( );

    /*  *** READ THIS ***
        I've added code here, once again, to monitor the interrupt code and
        adjust the duty cycle progressively up and down.  The starting idea
        begins with Project 10's approach -- the CCIFG1 event is used to
        toggle the LED.  But we've added a counter.  The idea here is that
        every two CCIFG1 events represents one LED cycle.  If that is true,
        then we can use four of them to represent two cycles.  The code here
        uses a loop that depends upon the counter value and waits until at
        least four CCIFG1 events happen.  Then the TACCR1 value, held in a
        variable called 'tac', is either increased or reduced depending upon
        another variable created, 'mode', which tells us which way to go.
        The code also includes tests for reaching the minimum and maximum
        limits for TACCR1 and if they are reached, switching direction.  So
        the overall idea appears to have merit.

        Note that we are back doing all this outside the interrupt code.  That
        means the main code isn't as free as we'd like, once again.  But it
        has the advantage of adding a feature without changing the interrupt
        code that existed in Project 10.  Sometimes, that's a good thing.
    */
        auto unsigned int tac= TACCR1;                  // *** FROM HERE ***
        for ( ; ; ) {
          static enum { brighter, dimmer } mode= dimmer;
            for ( icounter= 0; icounter < 4; )
                ;
            switch ( mode ) {

            case brighter:
                if ( tac < (TACADJ + PROXIMITY) ) {
                    tac= PROXIMITY;
                    mode= dimmer;
                } else
                    tac -= TACADJ;
                break;

            case dimmer:
                if ( tac > (TAC - TACADJ - PROXIMITY - 1) ) {
                    tac= TAC - PROXIMITY - 1;
                    mode= brighter;
                } else
                    tac += TACADJ;
                break;
            }
            TACCR1= tac;
        }                                               // *** TO HERE ***

    return 0;           // once again, this line produces a warning.
}



// #define     TAIFG_EVENT     0x0A
#define     CCIFG1_EVENT    0x02

/*  ----------------------------------------------------------------------  */
#pragma vector= TIMERA1_VECTOR
__interrupt void timer_A2_CCIFG1_TAIFG( void ) {
/*  ----------------------------------------------------------------------
    This routine reads the TAIV register to find out what caused the
    interrupt.  If it is a CCIFG1 compare-match event, then the LED is
    toggled to create the flicker effect.
*/
        switch ( TAIV ) {
        case CCIFG1_EVENT:
            P1OUT ^= 0x01;
        //    break;
        // case TAIFG_EVENT:
            ++icounter;
            break;
        }

    return;
}


#if 0
        __low_power_mode_0( );

    return 0;
}

#define     TAIFG_EVENT     0x0A
#define     CCIFG1_EVENT    0x02

/*  ----------------------------------------------------------------------  */
#pragma vector= TIMERA1_VECTOR
__interrupt void timer_A2_CCIFG1_TAIFG( void ) {
/*  ----------------------------------------------------------------------
    This routine reads the TAIV register to find out what caused the
    interrupt.  If it is a CCIFG1 compare-match event, then the LED is
    toggled to create the flicker effect.
*/
    static enum { brighter, dimmer } mode= dimmer;
    auto unsigned int tac;

        switch ( TAIV ) {
        case CCIFG1_EVENT:
            P1OUT ^= 0x01;
            break;
         case TAIFG_EVENT:
            tac= TACCR1;
            switch ( mode ) {
            case brighter:
                if ( tac < (TACADJ + PROXIMITY) ) {
                    tac= PROXIMITY;
                    mode= dimmer;
                } else
                    tac -= TACADJ;
                break;
            case dimmer:
                if ( tac > (TAC - TACADJ - PROXIMITY - 1) ) {
                    tac= TAC - PROXIMITY - 1;
                    mode= brighter;
                } else
                    tac += TACADJ;
                break;
            }
            TACCR1= tac;
            break;
        }

    return;
}
#endif
